/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher;

import com.nemmco.participantbatcher.process.BusinessProcess;
import com.nemmco.participantbatcher.process.CleanupProcess;
import com.nemmco.participantbatcher.process.ResendProcess;
import com.nemmco.participantbatcher.util.ApplicationException;
import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.MonitorLogger;
import com.nemmco.participantbatcher.util.PropertiesReader;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.File;

import java.text.SimpleDateFormat;

import java.util.Calendar;

import org.apache.log4j.Logger;

/**
 * This is the main class of the Instance Batcher application.
 * It starts all process threads. It will also be used as the
 * monitoring/control thread
 *
 * @author Kevin Jin
 * version 1.0 - 28 Aug 2003 - Creation
 * version 1.1 - 4 Nov 2003 modified logging, added version number, added loggerlevel
 * version 1.3 - 19-May-2004 added stop file processing, log4j
 * version 1.3.1 - 16-Nov-2004 fixed bug with the production of -ve ack files
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.1 - 09-Sep-2005 Modified the ack to make the ID used unique
 * version 1.4.2 - 17-Dec-2005 Not get caught by the filename.ack.tmp problem this is not an .ack file
 *                  -> added custom clean function;
 *                  -> added HttpServer function
 *                  -> free/total memory added to debug monitor message
 * version 1.4.3 - 22-Mar-2006 bug fixes of v1.4.2 - defects 3771,3774,3776
 * version 1.4.4 - 22-Apr-2006 bug fixes of v1.4.3 - SendProcess,FTPEDTProcesser,logging,javaDoc,coding standards
 * version 1.5.0 - 23-June-206 Created InstanceBatcher to replace functions of MeteringBatcher Mike Muir
 * version 1.5.2 - 13-Apr-2007 Fix sending of B2B to send any file that is schema invalid or zip invalid (so that it gets -ve acked), fixSpecialCodes as well
 * version 1.5.3 - 30-Jun-2007 fixed logger variable to be this class
 * version 2.0.0 - 15-Sep-2010 6356 Update version number to V2.0 for Java 1.6 upgrade.
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 */

public class InstanceBatcher {

   private static Logger logger = Logger.getLogger(InstanceBatcher.class);

   /**
    * Constructor
    */
   public InstanceBatcher() {}

   /** The main method of the Instance Batcher application.
    * This is the entry point of this application.
    *<p>
    * Set the monitor limit to > most other timers eg
    * processwaittime=1200, resendwaittime=1200, cleanwaittime=1200, exceptwaittime=1200
    * monitorwaittime=1000
    *<p>
    * Set the wait to stop to be a few minutes so that it only goes off if there is a failure eg
    * stopfilewaittime=300000
    *
    * @param args String[] instanceName participantid
    */
   public static void main(String[] args) {

      InstanceBatcher ib = new InstanceBatcher();
      String versionNumber = Constants.APPLICATION_VERSION;

      /* First, we have to retrieve all arguments
         * Note that there should be only 2 arguments - the instance name and participantID
         */
      String instanceName = "";
      String participantID = "";
      if (args.length != 2) {
         System.out.println("Usage: java -classpath \"......\" " + "com.nemmco.participantbatcher.InstanceBatcher " + "instanceName ParticipantID");
         System.exit(1);
      } else {
         instanceName = args[0];
         participantID = args[1];
      }

      // initialise
      Constants c = Constants.getConstant();
      try {

         // Now let's load properties from the instance's proerties file
         ib.loadProperties(instanceName);

         // fixup the constants(?) by replacing %participant% with the participantID
         Util_Logger.debug(logger, "Participant " + participantID + " substituted " + c.updateConstants(participantID) + " times");

         // put list of constants from properties onto log
         c.debugConstants();

         // Dont Start the shutdown hook
         Runtime rt = Runtime.getRuntime();

         // Start the main business processes
         BusinessProcess bp = new BusinessProcess();
         bp.start();

         // Start the resend business processes
         ResendProcess rp = new ResendProcess();
         if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(c.getProcessType())) {
            rp.start();
         }

         // Start the cleanup process
         CleanupProcess cp = new CleanupProcess();
         cp.start();

         // tell the user that startup has finished
         SimpleDateFormat dateFormat;
         dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

         System.out.println("" + dateFormat.format((Calendar.getInstance()).getTime()) + " Instance " + instanceName + " has started, version " + versionNumber);
         Util_Logger.info(logger, "Instance " + instanceName + "(" + participantID + ") has started, version " + versionNumber);

         /* The main thread of this application will
             * wait fior the monitor period before stopping - make sure that the delay times of the other
             * threads are greater than the monitor thread time so that they all just do one cycle and
             * are asked to stop as soon as they finish one cycle
             */
         //Thread t = new Thread();  // java 1.6
         String monitorFileName = c.getMonitorFileDir() + "/" + instanceName + "_monitor.log";
         File monitorFile = new File(monitorFileName);
         Thread.sleep(c.getMonitorWaitTime());

         // this checking of threads is probably useless in this application but cant hurt
         // test threads, if they are all alive, write a message to the monitor log
         boolean isAllAlive = false;

         if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(c.getProcessType())) {
            if (cp.isAlive() && bp.isAlive() && rp.isAlive()) {
               isAllAlive = true;
            }
         } else {
            if (cp.isAlive() && bp.isAlive()) {
               isAllAlive = true;
            }
         }

         // update the monitor file only if all threads ok and main process is connected
         if (isAllAlive && bp.getConnected()) {
            Util_Logger.debug(logger,
                              "Monitor thread -- all is ok. Free mem " + rt.freeMemory() / 1024 / 1024 + " Max mem " + rt.maxMemory() / 1024 / 1024 + " MB");
            MonitorLogger.log(monitorFile);
         } else if (isAllAlive) {
            Util_Logger.warning(logger, "Monitor thread -- no communication connection");
         } else {
            // this case is thread failed and possibly no communication connection
            Util_Logger.warning(logger, "Monitor thread -- one or more thread(s) are not alive - restart");
         }

         // stop the threads because the monitor timeout has happened
         if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(c.getProcessType())) {
            cp.requestStop();
            bp.requestStop();
            rp.requestStop();
         } else {
            cp.requestStop();
            bp.requestStop();
         }

         boolean isAnyAlive = false;
         int waitTime = 0;

         /**
             * check if the threads have stopped
             * loop if they havent stopped up to a wait time
             * set the stop wait time >>>> the cycle times so that the work
             * threads are not stopped in the middle of their work except if
             * a failure has occured eg lockup
            */
         do {
            isAnyAlive = false;
            if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(c.getProcessType())) {
               if (cp.isAlive() || bp.isAlive() || rp.isAlive()) {
                  isAnyAlive = true;
               }
            } else {
               if (cp.isAlive() || bp.isAlive()) {
                  isAnyAlive = true;
               }
            }
            // wait a second
            Thread.sleep(10);
            waitTime = waitTime + 10;

         } while (isAnyAlive && (c.getStopFileWaitTime() > waitTime));

         // keep users informed and stop
         System.out.println("" + dateFormat.format((Calendar.getInstance()).getTime()) + " Stopped instance ");

         // inform user of how the process is finishing and return to
         // the operating system JVM
         if (isAnyAlive) {
            Util_Logger.error(logger, "Some threads were terminated");
            System.exit(1);
         } else {
            Util_Logger.info(logger, "Stopped all threads correctly");
            System.exit(0);
         }

      } catch (Exception e) {
         if (e instanceof ApplicationException) {

            // Fatal error occured during the application running
            ApplicationException ae = (ApplicationException)e;
            System.out.println("Application error happened when running this program, Please check the log file");
            if (!ae.getLogID().isEmpty()) {
               System.out.println("The error ID is: " + ae.getLogID());
            }
            System.out.println("The error is: " + ae.getErrorMessage());
         } else {

            // Can not start the application
            System.out.println("Error happened when starting this application, Please try again.");
            System.out.println("If problem persist, Please inform technical support, and quote below.");
            System.out.println("Exception was " + e.getMessage());
            e.printStackTrace();
         }

      }
   }

   // Load properties
   private void loadProperties(String instanceName) throws Exception {
      //PropertiesReader pr = new PropertiesReader(); // java 1.6
      PropertiesReader.readProperties(instanceName);
   }
}
