/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.ftp;

import java.io.File;

/**
 * This Interface is used to define all methods required for FTP processing.
 *
 * The purpose of this Interface is to standardize FTP processes and allows
 * easy conversion to use other FTP third party libraries in the future.
 *
 * @author Kevin Jin
 * version 1.0 - 21 Aug 2003 - Creation
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.4 - 6-Apr-2006 Added call to set port range for use on difficult sites
 *                  and with edtftpj, coding standards, javaDoc,
 * @version 1.5.0 - 07-Aug-2006 added domain for webDAV
 *
 */

public interface ComProcess {

   /**
    * Connect to the FTP server.
    * <p>
    * @param serverName String
    * @exception ComException If an error occurs while connecting.
    */
   public void connect(String serverName) throws ComException;

   /**
    * Set the port working range in active ftp mode.  You need to know
    * exactly what you are doing to use this correctly.
    * <p>
    * @param lowLimit int
    * @param highLimit int
    * @exception ComException If an error occurs while setting range
    */
   public void setActivePortRange(int lowLimit, int highLimit) throws ComException;

   /**
    * Set the domain for webDAV
    *<p>
    * @param domain String
    * @exception ComException if an error occurs
    */
   public void setDomain(String domain) throws ComException;

   /**
    * Closes the connection to the FTP server.
    * <p>
    * @exception ComException If an error occurs while disconnecting.
    */
   public void disconnect() throws ComException;

   /**
    * Test if the FTP connection is still valid.
    * <p>
    * @return boolean true - if the FTP connection is still valid
    * @exception ComException If an error occurs while disconnecting.
    */
   public boolean isConnected() throws ComException;

   /**
    * Login to the FTP server using the provided username and password.
    * <p>
    * @param userName The username to login under.
    * @param password The password to use.
    * @return boolean true - if successfully completed, false if not.
    * @exception ComException  If an error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    */
   public boolean login(String userName, String password) throws ComException;

   /**
    * Logout of the FTP server by sending the QUIT command.
    * <p>
    * @return boolean true if successfully completed, false if not.
    * @exception ComException  If an error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    */
   public boolean logout() throws ComException;

   /**
    * Test if the reply from FTP server is positive.
    * <p>
    * @return boolean TRUE if positive reply, FALSE if negative reply
    * @exception ComException If an error occurs while disconnecting.
    */
   public boolean isPositiveReply() throws ComException;

   /**
    * Sets the timeout in milliseconds to use when processing (downloading
    * and uploading) files.  This timeout should be set immediately after
    * opening the FTP connection. and the value should be able to be set
    * in the configuration file.
    *
    * @param  timeout int The default timeout in milliseconds that is used when
    *         processing a file.
    */
   public void setDataTimeout(int timeout);

   /**
    * Change the current working directory of the FTP session.
    *<p>
    * @param pathName  The new current working directory.
    * @return boolean true if successfully completed, false if not.
    * @exception ComException  If an error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    */
   public boolean changeWorkingDirectory(String pathName) throws ComException;

   /**
    * Sets the file type to be transferred.
    * This should be one of ASCII, BINARY for Participant Batcher
    * The file type only needs to be set when you want to change the type.
    * After changing it, the new type stays in effect until you change it
    * again.  The default file type is ASCII
    * <p>
    * @param      fileType either ASCII or BINARY indcating the type of file.
    * @return     boolean true if successfully completed, false if not.
    * @exception  ComException  If an error occurs while either sending a
    *             command to the server or receiving a reply from the server.
    */
   public boolean setFileType(String fileType) throws ComException;

   /**
    * Retrieves a named file from the server and writes it to the given file.
    * <p>
    * @param remote  The name of the remote file.
    * @param local   The local file to which to write.
    * @return boolean true if successfully completed, false if not.
    * @exception ComException  If an error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    */
   public boolean retrieveFile(String remote, File local) throws ComException;

   /**
    * Stores a file on the server using the given name and taking input
    * from the given file.
    * <p>
    * @param remote  The name to give the remote file.
    * @param local   The local file from which to read.
    * @return boolean true if successfully completed, false if not.
    * @exception ComException  If an I/O error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    */
   public boolean storeFile(String remote, File local) throws ComException;

   /**
    * Renames a remote file.
    * <p>
    * @param from  The name of the remote file to rename.
    * @param to    The new name of the remote file.
    * @return boolean true if successfully completed, false if not.
    * @exception ComException  If an I/O error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    */
   public boolean rename(String from, String to) throws ComException;

   /**
    * Abort a transfer in progress.
    * <p>
    * @return boolean true if successfully completed, false if not.
    * @exception ComException  If an I/O error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    */
   public boolean abort() throws ComException;

   /***
    * Deletes a file on the FTP server.
    * <p>
    * @param pathname   The pathname of the file to be deleted.
    * @return boolean true if successfully completed, false if not.
    * @exception ComException  If an I/O error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    ***/
   public boolean deleteFile(String pathname) throws ComException;

   /***
    * Obtain a list of filenames in a directory.
    * If the given pathname contains no files,  a zero length array or null is
    * returned. If the directory is not empty, an array of filenames in the
    * directory is returned. If the pathname corresponds to a file, only that
    * file will be listed.
    * <p>
    * @param pathName  The directory or file to list.
    *                  NULL or empty string stands for current working directory
    * @return The list of filenames contained in the given path.  null if
    *     the list could not be obtained.  If there are no filenames in
    *     the directory, a zero-length array is returned.
    * @exception ComException  If an error occurs while either sending a
    *      command to the server or receiving a reply from the server.
    ***/
   public String[] listNames(String pathName) throws ComException;

}
