/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.ftp;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.Util_Logger;

import org.apache.log4j.Logger;

/**
 * This class implements the singleton pattern.
 * <p>
 * It creates and manages FTP clients -- FTPProcesser classes for
 * both the main thread and resend thread.
 * <p>
 * @author Kevin Jin
 * version 1.0 - 21 Aug 2003 - Kevin Jin - Creation
 * version 1.3 - 04-Oct-2004 log4j, removed Resend FTP
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.4 - 24-Apr-2006 Added a new 3rd party FTP client - EDT - and made easier to add further clients
 *                              Set timeout time before making connection, fix disconnect on set to NULL,
 *                              Change loggin on disconnections after connection failures, coding standards, javaDoc,
 * version 1.5.3 - 19-Jul-2007 Make the return of instance thread safe
 * version 2.0.1 - 15-Nov-2010 Removed WebDAV as it does not work
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 */

public class ComProcessManager {

   // The only instance of the ComProcessManager object
   private static ComProcessManager comProcessManager = null;

   // The ComProcesser instance for the main thread
   private ComProcess mainCom = null;

   // Logger definition
   private static Logger logger = Logger.getLogger(ComProcessManager.class);

   /** Constructor */
   private ComProcessManager() {}

   /**
    * This class is used to set the only copy of this object.
    */
   private static class SingletonComProcessManagerHolder {

      private static ComProcessManager instance = new ComProcessManager();
   }

   /**
    * Get the only instance of the ComProcessManager object.
    *
    * @return ComProcessManager the only instance of the ComProcessManager
    */
   public static ComProcessManager getInstance() {
      if (comProcessManager == null) {
         comProcessManager = SingletonComProcessManagerHolder.instance; // thread safe
      }
      return comProcessManager;
   }

   /**
    * Get the ready to use ComProcesser instance for the main thread.
    * <p>
    * @return ComProcess the Processer instance for the main thread.
    */
   public ComProcess getMainProcess() throws ComException {
      // first get the constants
      Constants c = Constants.getConstant();
      String server = c.getFTPServerName();
      String domain = c.getDomain();
      String username = c.getUserName();
      String password = c.getPassword();
      int lowLimit = c.getLowFtpPort();
      int highLimit = c.getHighFtpPort();

      // then get the configured processer  // thread safe
      if ((mainCom == null) || !(mainCom.isConnected())) {

         if (c.getComMode().equalsIgnoreCase(Constants.COM_MODE_FTP)) {
            mainCom = new FTPProcesser();
         } else if (c.getComMode().equalsIgnoreCase(Constants.COM_MODE_FTP_EDT)) {
            mainCom = new FTPEDTProcesser();
         } else if (c.getComMode().equalsIgnoreCase(Constants.COM_MODE_SHARE)) {
            mainCom = new ShareProcesser();
            server = c.getPNInboxDir();
         }

         // connect and login
         try {

            Util_Logger.debug(logger, "Start connecting to " + server);
            mainCom.setDataTimeout(c.getTimeout());
            mainCom.setDomain(domain);
            mainCom.connect(server);
            if (!mainCom.isPositiveReply()) {
               mainCom.disconnect();
               Util_Logger.warning(logger, "" + server + "refused connection. negative reply");
               throw new ComException("" + server + "refused connection. negative reply", null);
            }
            Util_Logger.debug(logger, "Successfully connected to server " + server);

            // v1.4.4 set active port range for edtFTPj client library if active and non-zero specified
            if ((lowLimit > 0) && (highLimit > 0)) {
               mainCom.setActivePortRange(lowLimit, highLimit);
               Util_Logger.debug(logger, "Port limits set from " + lowLimit + " to " + highLimit);
            }

            Util_Logger.debug(logger, "Start to log into the server");
            boolean status = mainCom.login(username, password);
            if (!status) {
               Util_Logger.warning(logger, "Can not log into the server");
               mainCom.logout();
               throw new ComException("Can not log into the server", null);
            }
            Util_Logger.info(logger, "Logged into the server " + server);

         } catch (ComException e) {
            if (mainCom.isConnected()) {
               mainCom.disconnect();
            }
            throw e;
         } catch (Exception e) {
            if (mainCom.isConnected()) {
               mainCom.disconnect();
            }
            throw new ComException(e.getMessage(), e);
         }
      }

      return mainCom;
   }

   /**
    * Set FTPProcesser instance for the main thread to NULL because of error.
    * <p>
    */
   public void setMainProcessToNULL() {

      // need to disconnect to free up control channel if FTP
      if (mainCom != null) {

         try {
            if (mainCom.isConnected()) {
               mainCom.disconnect();
            }
         } catch (Exception f) {
            ;
         } // v1.4.4
      }
      mainCom = null;
   }
}
