/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 * 10-Feb-2015     Warwick Mills      5252 Metering Batcher fails when duplicate file exists
 */

package com.nemmco.participantbatcher.process;

import com.nemmco.participantbatcher.util.CleanFileFilter;
import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.CustomClean;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.File;
import java.io.FilenameFilter;

import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * This class is used to run the cleanup process thread.  The cleanup process
 * moves file from the persistance directories (pli and plo) to the various
 * sub directories off pli and plo after the process is finished.  It also allowas the user
 * to move files from these sub-directories to other places as desired by the user.
 *
 * @author Kevin Jin
 *  version 1.0   - 27 Aug 2003 - Creation
 *  version 1.1   -  4 Nov 2003 modified logging
 *  version 1.3   - 19-May-2004 added stop file processing, 21-Aug-2004 renamed files to becom .ack or .zip finally, log4j
 *  version 1.4.2 - 17-Dec-2005 Added a custom clean function to move files anywhere, reduced debug messages
 *  version 1.4.4 -  6-May-2006 Fixed javaDoc and coding standards
 *  version 1.5.1 -  1-Nov-2006 Made BatcherFileFilter generic to handle a file type or a set of file types, replaces AckFileFilter
 *                              added partSleep to improve reponsiveness when stopping
 *  version 2.0.3 - 22-Feb-2011 Ensure application exception kills thread to signal application failure
 * @version 2.0.7 - 10-Feb-2015 Overwrite duplicate files
 */

public class CleanupProcess extends PbThread {

   private static Logger logger = Logger.getLogger(CleanupProcess.class);

   /**
    * Constructor
    */
   public CleanupProcess() {}

   /**
    * Thread run method
    */
   public void run() {

      while (!getStop()) {
         long waitingTime = Constants.getConstant().getCleanWaitTime();

         try {
            Util_Logger.debug(logger, "Clean up process started");

            // copy all files from participant local inbox to its sub-directories
            long countFiles = copyInboxDones() + copyInboxNacks() + copyInboxResends();

            if (getStop()) {
               break;
            }

            // copy all files from participant local outbox to its sub-directories
            countFiles = countFiles + copyOutboxNacks() + copyOutboxZips();

            long moveFiles = 0;
            Vector<CustomClean> v = Constants.getConstant().getCustomClean();
            for (int i = 0; i < v.size(); i++) {
               if (getStop()) {
                  break;
               }
               moveFiles = moveFiles + moveFiles(v.elementAt(i));
            }

            if (countFiles + moveFiles > 0) {
               Util_Logger.info(logger, "End Clean " + countFiles + " local files moved, " + moveFiles + " remote moved");
            } else {
               Util_Logger.debug(logger, "End Clean 0 local files moved, 0 remote moved");
            }

         } catch (Exception e) {
            // there is only one possibility
            // ApplicationException -- Local file process failure
            Util_Logger.warning(logger, "Error occured when running clean up process");
            //waitingTime = 0;  v 2.0.3
         }

         // go to sleep for user specified timeframe
         partSleep(waitingTime);
      }

   }

   // Copy .done file from participant local inbox to its done sub-directory
   private long copyInboxDones() {

      String fromDir = Constants.getConstant().getPLInboxDir();
      String toDir = fromDir + "/done";
      BatcherFileFilter filter = new BatcherFileFilter(".done");
      //Util_Logger.debug(logger,"Start to copy inbox DONE files to its subdir done");
      long i = cleanupFiles(fromDir, toDir, filter);
      this.renameFiles(toDir, "zip", filter); // *.done -> *.zip
      this.renameFiles(toDir, "ack", new BatcherFileFilter(new String[] { ".ack_done" })); // *.ack_done -> *.ack
      //Util_Logger.debug(logger,"Finish to copy inbox DONE files to its subdir done");
      return i;
   }

   // Copy .nack file from participant local inbox to its nack sub-directory
   private long copyInboxNacks() {

      String fromDir = Constants.getConstant().getPLInboxDir();
      String toDir = fromDir + "/nack";
      BatcherFileFilter filter = new BatcherFileFilter(".nack");
      //Util_Logger.debug(logger,"Start to copy inbox NACK files to its subdir nack");
      long i = cleanupFiles(fromDir, toDir, filter);
      this.renameFiles(toDir, "zip", filter); // *.nack -> *.zip
      this.renameFiles(toDir, "ack", new BatcherFileFilter(new String[] { ".ack_nack" })); // *.ack_nack -> *.ack
      //Util_Logger.debug(logger,"Finish to copy inbox NACK files to its subdir nack");
      return i;
   }

   // Copy .resend file from participant local inbox to its resend sub-directory
   private long copyInboxResends() {

      String fromDir = Constants.getConstant().getPLInboxDir();
      String toDir = fromDir + "/resend";
      BatcherFileFilter filter = new BatcherFileFilter(".resend");
      //Util_Logger.debug(logger,"Start to copy inbox RESEND files to its subdir resend");
      long i = cleanupFiles(fromDir, toDir, filter);
      //Util_Logger.debug(logger,"Finish to copy inbox RESEND files to its subdir resend");
      return i;
   }

   // Copy .nack file from participant local outbox to its nack sub-directory
   private long copyOutboxNacks() {

      String fromDir = Constants.getConstant().getPLOutboxDir();
      String toDir = fromDir + "/nack";
      BatcherFileFilter filter = new BatcherFileFilter(".nack");
      //Util_Logger.debug(logger,"Start to copy outbox NACK files to its subdir nack");
      long i = cleanupFiles(fromDir, toDir, filter);
      this.renameFiles(toDir, "zip", filter);
      //Util_Logger.debug(logger,"Finish to copy outbox NACK files to its subdir nack");
      return i;
   }

   // Copy .zip file from participant local outbox to its zip sub-directory
   private long copyOutboxZips() {

      String fromDir = Constants.getConstant().getPLOutboxDir();
      String toDir = fromDir + "/zip";
      BatcherFileFilter filter = new BatcherFileFilter(".zip");
      //Util_Logger.debug(logger,"Start to copy outbox ZIP files to its subdir zip");
      long i = cleanupFiles(fromDir, toDir, filter);
      //Util_Logger.debug(logger,"Finish to copy outbox ZIP files to its subdir zip");
      return i;
   }

   // Copy files from one directory to another
   private long cleanupFiles(String fromDir, String toDir, FilenameFilter filter) {

      FileManager fm = new FileManager();
      String[] files = fm.scanDir(fromDir, filter);
      long countFiles = 0;

      if ((files != null) && (files.length > 0)) {

         Util_Logger.debug(logger, "Cleanup. " + files.length + " files will be copied From " + fromDir + " To " + toDir + " Filter " + filter.toString());

         for (int i = 0; i < files.length; i++) {
            String file = files[i];
            if (fm.copyLocalFile(fromDir, toDir, file)) {
               String message = "Cleanup. Copy file " + file + " from " + fromDir + " to " + toDir + " succeed";
               Util_Logger.debug(logger, message);
               countFiles++;
            }
         }
         //Util_Logger.debug(logger,"Cleanup. " + files.length + " files have been copied");

      } else { // there is no files to process
         //Util_Logger.debug(logger,"Cleanup. No file will be copied");
      }
      return countFiles;
   }

   // Copy files from one directory to another
   private long renameFiles(String toDir, String toExtension, FilenameFilter filter) {

      FileManager fm = new FileManager();
      String[] files = fm.scanDir(toDir, filter);
      long countFiles = 0;

      if ((files != null) && (files.length > 0)) {

         Util_Logger.debug(logger, "Cleanup. " + files.length + " files will be renamed In " + toDir + " Ext " + toExtension + " Filter " + filter.toString());

         for (int i = 0; i < files.length; i++) {
            String file = files[i];
            String newFileName = fm.replaceFileName(file, toExtension);
            if (fm.renameFile(toDir, file, newFileName)) {
               String message = "Cleanup. Rename file " + file + " in " + toDir + " succeed";
               Util_Logger.debug(logger, message);
               countFiles++;
            }
         }
         //Util_Logger.debug(logger,"Cleanup. " + files.length + " files have been renamed");

      } else { // there is no files to process
         //Util_Logger.debug(logger,"Cleanup. No file will be renamed");
      }
      return countFiles;
   }

   // Move files from one directory to another
   private long moveFiles(CustomClean customClean) {

      int cleanedFiles = 0;
      File file = null;
      String fromDir = customClean.getFromDir();
      String toDir = customClean.getToDir();
      String[] filenames = null;
      FileManager fm = new FileManager();

      // 5252 - delete aged files from custom clean target
      long cleanAge = Constants.getConstant().getCleanAge();
      if (cleanAge > 0L) {
         long ageLimit = System.currentTimeMillis() - (cleanAge * 86400000L);
         filenames = fm.scanDir(toDir, null);
         if ((filenames != null) && (filenames.length > 0)) {
            for (String filename : filenames) {
               file = new File(toDir + "/" + filename);
               if (file.lastModified() < ageLimit) {
                  if (fm.deleteFile(toDir + "/" + filename))
                     cleanedFiles++;
               }
            }
            if (cleanedFiles > 0) {
               Util_Logger.info(logger, "Custom Cleanup removed " + cleanedFiles + " aged files from " + toDir);
            }
         }
      }

      CleanFileFilter filter = new CleanFileFilter(customClean);
      filenames = fm.scanDir(fromDir, filter);
      long countFiles = 0;

      if ((filenames != null) && (filenames.length > 0)) {

         Util_Logger.debug(logger, "Custom Cleanup. " + filenames.length + " files will be moved " + toDir + " filter " + filter.toString());

         for (String filename : filenames) {
            if (fm.renameFile(fromDir, filename, toDir, filename)) {
               String message = "Cleanup. Move file " + filename + " in " + toDir + " succeeded";
               Util_Logger.debug(logger, message);
               countFiles++;
            }
         }
         //Util_Logger.debug(logger,"Cleanup. " + filenames.length + " files have been renamed");

      } else { // there is no files to process
         //Util_Logger.debug(logger,"Cleanup. No file will be renamed");
      }
      return countFiles;
   }
}
