/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.process;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import java.util.Properties;

import org.apache.log4j.Logger;

/**
 * This class is used to control file resend tries, which will be stored
 * in a control file.
 * <p>
 * @author Kevin Jin
 * version 1 (18/09/2003) -- Creation
 * version 1.3 04-Oct-2004 log4j
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * @version 2.0.2 19-Nov-2010 Peer review work
 */
public class ResendTryController {

   // The only instance of this class
   private static ResendTryController theInstance = null;

   private String controlFileName = "";
   private Properties controller = null;
   private static Logger logger = Logger.getLogger(ResendTryController.class);

   // This class implements the Singlton pattern, therefore, the
   // constructor is defined as private
   private ResendTryController(String instanceName) {
      controlFileName = Constants.getConstant().getResendTryControlFileDir() + "/" + instanceName + ".control";
      populateInstance();
   }

   /**
    * This method is used to get the only object of this ResendTryController class.
    *
    * @param instanceName - this application instance name String
    * @return ResendTryController - the only object of this class  // thread safe ??
    */
   public static ResendTryController getInstance(String instanceName) {
      if (theInstance == null) {
         theInstance = new ResendTryController(instanceName);
      }
      return theInstance;
   }

   /**
    * This method is used to get the resend try number for specified file.
    *
    * @param fileName String
    * @return int try number for this file
    */
   public int getTryNumber(String fileName) {
      int ret = 0;
      if (controller.containsKey(fileName)) {
         String number = (String)controller.get(fileName);
         try {
            ret = Integer.parseInt(number);
         } catch (Exception e) {
            // this should not happened, if happened,
            // we return the max number to prevent load this file
            ret = Constants.getConstant().getResendTryControlNo();
         }
      }
      return ret;
   }

   /**
    * This method is used to update the resend try control number
    * for specified file.
    *
    * @param fileName String
    * @param newNumber int the new resend try number
    */
   public void updateTryNumber(String fileName, int newNumber) {

      String number = Integer.toString(newNumber);
      if (controller.containsKey(fileName)) {
         controller.remove(fileName);
      }
      controller.put(fileName, number);
      storeInstance();

   }

   /**
    * This method is used to delete the entry in the resend try control file
    * for specified file.
    *
    * @param fileName String
    */
   public void deleteEntry(String fileName) {

      if (controller.containsKey(fileName)) {
         controller.remove(fileName);
         storeInstance();
      }

   }

   // this method is used to populate the hash table from the file
   private void populateInstance() {

      // create a properties file
      controller = new Properties();
      try {

         // First, load the file into a Properties object
         File controlFile = new File(controlFileName);
         if (controlFile.exists()) {
            FileInputStream inStream = new FileInputStream(controlFile);
            try {
               controller.load(inStream);
            } finally {
               inStream.close();
            }
         }
      } catch (Exception e) {
         Util_Logger.warning(logger, "Can not load data from resend try control file", e);
      }
   }

   // this method is used to store data in the controller property back to the
   // resend try control file
   private void storeInstance() {

      try {
         File controlFile = new File(controlFileName);
         if (!controlFile.exists()) {
            if (!controlFile.createNewFile())
               throw new Exception("Create of retry control file failed " + controlFileName);
         }
         FileOutputStream outStream = new FileOutputStream(controlFile);
         try {
            controller.store(outStream, "");
         } finally {
            outStream.close();
         }
      } catch (Exception e) {
         Util_Logger.warning(logger, "Can not store data into resend try control file", e);
      }
   }
}
