/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 * 10-Feb-2015     Warwick Mills      5252 Metering Batcher fails when duplicate file exists
 * 06-Mar-2019     Deepak Gupta       DFCT0022009
 */

package com.nemmco.participantbatcher.util;

import java.io.BufferedInputStream;

import java.net.URL;
import java.net.URLConnection;

import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * This class is used to retrieve information from properties file.
 * It will populate the singleton-patterened Constants class
 * <p>
 * author Kevin Jin
 * version 1.0 - 27 Aug 2003 - Creation
 * version 1.1 4 Nov 2003 added exceptWaitTime, logginglevel
 * version 1.3 - 19-May-2004 added stopFilePath, stopFileWait, added log4j, instanceName
 * version 1.3.3 - 16-feb-2005 added ftpmode
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.2 - 17-Dec-2005 Adding a custom clean file feature (buildVector); port
 *                - Added a trim on property values to remove trailing spaces
 * version 1.4.3 - 22-Mat-2006 Defect 3774 a null file mask in custom clean and in fileTypes... should be ignored
 * version 1.4.4 - 24-Apr-2006 Added new 3rd party FTP client EDT, cleaned up logging for bad properties
 * version 1.5.0 - 10-Jun-2006 Added webDAV com process
 * version 1.5.1 - 03-Nov-2006 Made processtype if illegal to give a warning in the log, changed custom clean definition
 * version 1.5.2 - 22-Mar-2007 List out the classpath if properties file URL not found
 * version 1.5.3 - 15-Jul-2009 5796 Store B2BFileTypes as lower case for later comparison.
 * version 1.5.4 - 18-Aug-2007 Allow a separate log4j properties file
 * version 2.0.1 - 15-Nov-2010 Peer review work
 *  version 2.0.2 - 19-Nov-2010 Peer review work
 * version 2.0.7 - 10-Feb-2015 Overwrite duplicate files
 */

public class PropertiesReader {

   private static Logger logger = Logger.getLogger(PropertiesReader.class);

   /**
    * Constructor
    */
   public PropertiesReader() {}

   /**
    * Convert a Url that points to a property file into a loaded properties object
    * @param url - the Url pointing to the property information
    * @throws Exception
    * @returns - the properties object
    */
   private static Properties getProperties(URL url) throws Exception {

      // connect to url
      URLConnection conn = url.openConnection();
      conn.connect();
      Properties properties = new Properties();
      BufferedInputStream data = new BufferedInputStream(conn.getInputStream());

      try {
         // load the file into a Properties object
         properties.load(data);
      } finally {
         data.close();
      }

      return properties;
   }

   /**
    * This mehods is used to retrieve properties information
    * from the NAME.properties file.
    * <p>
    * @param name - the name of the properties file
    * @throws Exception - any error occured during the loading properties process
    */
   public static void readProperties(String name) throws Exception {

      // store instance name
      Constants.getConstant().setInstanceName(name);

      // find the properties file
      String propertiesFileName = name + ".properties";
      URL url = null;
      url = PropertiesReader.class.getClassLoader().getResource(propertiesFileName);

      if (url == null) {
         String msg = "Cannot find " + propertiesFileName + ", " + "Please check your class path settings " + System.getProperty("java.class.path");
         throw new Exception(msg);
      }

      // load the properties
      Properties properties = getProperties(url);

      // setup the log4j logger first  - from this point on can use the logger
      URL urlLog4j = null;
      urlLog4j = PropertiesReader.class.getClassLoader().getResource("log4j.properties");
      if (urlLog4j == null) {

         // log4j is in the instance properties file, extract it and configure logger
         PropertyConfigurator.configure(properties);

      } else {

         // log4j.properties file exists, extract it and configure logger
         Properties propertiesLog4j = getProperties(urlLog4j);
         PropertyConfigurator.configure(propertiesLog4j);

      }

      // Starting message to logger that should now exist
      Util_Logger.info(logger, "Instance " + name + " starting");

      String pName = "";
      String pValue = "";

      // Now, let go through all properties for Participant Batcher
      Enumeration<?> names = properties.propertyNames();
      while (names.hasMoreElements()) {

         pName = (String)names.nextElement();
         pValue = properties.getProperty(pName).trim();

         if ((pValue == null) || !(pValue.length() > 0)) {
            String msg = "Cannot load properties. Property - " + pName + " has invalid value";
            Util_Logger.error(logger, msg);
         }
         if ("ftpservername".equalsIgnoreCase(pName)) {
            Constants.getConstant().setFTPServerName(pValue);
         } else if ("username".equalsIgnoreCase(pName)) {
            Constants.getConstant().setUserName(pValue);
         } else if ("password".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPassword(pValue);
         } else if ("domain".equalsIgnoreCase(pName)) {
            Constants.getConstant().setDomain(pValue); // v1.5.0 webDAV
            // V1.3.3 ftpmode must be active or passive
         } else if ("ftpmode".equalsIgnoreCase(pName)) {

            if ("active".equalsIgnoreCase(pValue)) {
               Constants.getConstant().setFtpPassiveMode(false);
            } else if ("passive".equalsIgnoreCase(pValue)) {
               Constants.getConstant().setFtpPassiveMode(true);
            } else {
               String msg = "property - ftpmode must be either active or passive";
               Util_Logger.error(logger, msg);
            }

         }

         // V1.4.0,1.4.4 communication mode
         else if ("commode".equalsIgnoreCase(pName)) {

            if (Constants.COM_MODE_FTP.equalsIgnoreCase(pValue)) {
               Constants.getConstant().setComMode(Constants.COM_MODE_FTP);
            } else if (Constants.COM_MODE_SHARE.equalsIgnoreCase(pValue)) { // dont tell participants about this option
               Constants.getConstant().setComMode(Constants.COM_MODE_SHARE);
            } else if (Constants.COM_MODE_FTP_EDT.equalsIgnoreCase(pValue)) {
               Constants.getConstant().setComMode(Constants.COM_MODE_FTP_EDT);
            } else if (Constants.COM_MODE_WEBDAV.equalsIgnoreCase(pValue)) {
               Constants.getConstant().setComMode(Constants.COM_MODE_WEBDAV);
            } else {
               String msg = "property - commode is '" + pValue + "' must one of " + Constants.COM_MODE_FTP + "," + Constants.COM_MODE_FTP_EDT + "," + Constants.COM_MODE_WEBDAV;
               Util_Logger.error(logger, msg);
            }

         } else if ("lowftpport".equalsIgnoreCase(pName)) {
            Constants.getConstant().setLowFtpPort(checkInt(pName, pValue, 0, 65535));
         } else if ("highftpport".equalsIgnoreCase(pName)) {
            Constants.getConstant().setHighFtpPort(checkInt(pName, pValue, 0, 65535));
         } else if ("encryptedpassword".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPassword(Crypt.decrypt(pValue));

         } else if ("processtype".equalsIgnoreCase(pName)) {

            if ("1".equalsIgnoreCase(pValue)) {
               Constants.getConstant().setProcessType(Constants.PROCESS_SEND);
            } else if ("2".equalsIgnoreCase(pValue)) {
               Constants.getConstant().setProcessType(Constants.PROCESS_RECEIVE);
            } else if ("3".equalsIgnoreCase(pValue)) {
               Constants.getConstant().setProcessType(Constants.PROCESS_BOTH);
            } else {
               // Always default to running both processes
               String msg = "property - processtype is '" + pValue + "' must one of 1,2 or 3 - set to 3 in this case";
               Util_Logger.warning(logger, msg);
               Constants.getConstant().setProcessType(Constants.PROCESS_BOTH);
            }

         } else if ("filetypes".equalsIgnoreCase(pName)) {
            Constants.getConstant().setFileType(buildVector(pValue));
         } else if ("b2bfiletypes".equalsIgnoreCase(pName)) {
            Constants.getConstant().setB2BFileType(buildVector(pValue.toLowerCase()));
			} else if ("acceptTxTypes".equalsIgnoreCase(pName)) {
				Constants.getConstant().setAcceptTxTypes(buildVector(pValue));
			} else if ("rejectTxTypes".equalsIgnoreCase(pName)) {
				Constants.getConstant().setRejectTxTypes(buildVector(pValue));
			} else if ("acceptSchemas".equalsIgnoreCase(pName)) {
				Constants.getConstant().setAcceptSchemas(buildVector(pValue));
			} else if ("rejectSchemas".equalsIgnoreCase(pName)) {
				Constants.getConstant().setRejectSchemas(buildVector(pValue));

			} else if ("schemavalidation".equalsIgnoreCase(pName)) {
            if ("true".equalsIgnoreCase(pValue)) {
               Constants.getConstant().setValidateSchema(true);
            } else {
               Constants.getConstant().setValidateSchema(false);
            }

         } else if ("inboxhighmark".equalsIgnoreCase(pName)) {
            Constants.getConstant().setInboxHighMark(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("inboxlowmark".equalsIgnoreCase(pName)) {
            Constants.getConstant().setInboxLowMark(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("outboxhighmark".equalsIgnoreCase(pName)) {
            Constants.getConstant().setOutboxHighMark(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("outboxlowmark".equalsIgnoreCase(pName)) {
            Constants.getConstant().setOutboxLowMark(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("filewaittime".equalsIgnoreCase(pName)) {
            Constants.getConstant().setFileWaitTime(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("processwaittime".equalsIgnoreCase(pName)) {
            Constants.getConstant().setProcessWaitTime(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("exceptwaittime".equalsIgnoreCase(pName)) {
            Constants.getConstant().setExceptWaitTime(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("resendwaittime".equalsIgnoreCase(pName)) {
            Constants.getConstant().setResendWaitTime(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("cleanwaittime".equalsIgnoreCase(pName)) {
            Constants.getConstant().setCleanupWaitTime(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("monitorwaittime".equalsIgnoreCase(pName)) {
            Constants.getConstant().setMonitorWaitTime(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("timeout".equalsIgnoreCase(pName)) {
            Constants.getConstant().setTimeout(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("buffersize".equalsIgnoreCase(pName)) {
				Constants.getConstant().setBufferSize(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
			} else if ("fileoverwrite".equalsIgnoreCase(pName)) {
				Constants.getConstant().setFileOverwrite(Boolean.parseBoolean(pValue));
         } else if ("cleanage".equalsIgnoreCase(pName)) {
            Constants.getConstant().setCleanAge(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("pninbox".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPNInboxDir(pValue);
         } else if ("pnoutbox".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPNOutboxDir(pValue);
         } else if ("pnstopbox".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPNStopboxDir(pValue);
         } else if ("plinbox".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPLInboxDir(pValue);
         } else if ("ploutbox".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPLOutboxDir(pValue);
         } else if ("stopfiledir".equalsIgnoreCase(pName)) {
            Constants.getConstant().setStopFileDir(pValue);
         } else if ("stopfilewaittime".equalsIgnoreCase(pName)) {
            Constants.getConstant().setStopFileWaitTime(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("monitordir".equalsIgnoreCase(pName)) {
            Constants.getConstant().setMonitorFileDir(pValue);
         } else if ("resendcontroldir".equalsIgnoreCase(pName)) {
            Constants.getConstant().setResendTryControlFileDir(pValue);
         } else if ("resendcontrolno".equalsIgnoreCase(pName)) {
            Constants.getConstant().setResendTryControlNo(checkInt(pName, pValue, 0, Integer.MAX_VALUE));
         } else if ("port".equalsIgnoreCase(pName)) {
            Constants.getConstant().setPort(checkInt(pName, pValue, 0, 65535));
         } else if ("ackschemanamespace".equalsIgnoreCase(pName)) {
            Constants.getConstant().setAckSchemaNamespace(pValue);
         } else if ("ackschemalocation".equalsIgnoreCase(pName)) {
            Constants.getConstant().setAckSchemaLocation(pValue);
         } else if (pName.toLowerCase().startsWith("schemacache")) {
            Constants.getConstant().addSchemaCache(pValue);

         } else if (pName.toLowerCase().startsWith("customclean")) { // v1.4.2
            Vector<String> work = new Vector<String>();
            Vector<String> work2 = new Vector<String>();
            CustomClean customClean = new CustomClean();
            work = buildVector(pValue); // this contains the separate list of elements FromDir ToDir FileType Mask1 Mask2 ....

            // load into the custom clean object
            if (work.size() >= 4) {

               customClean.setFromDir(work.elementAt(0));
               customClean.setToDir(work.elementAt(1));
               customClean.setFileExtension(work.elementAt(2));

               // move the one or more masks
               for (int i = 3; i < work.size(); i++) {
                  work2.add(work.elementAt(i));
               }
               customClean.setFileMasks(work2);

               // store
               Constants.getConstant().addCustomClean(customClean);

            } else {
               String msg = "property - customclean must have 4 elements FromDir;ToDir;fileType;Mask;...";
               Util_Logger.error(logger, msg);
            }
         }

         else if (pName.toLowerCase().startsWith("log4j")) {
            //skip log4j included properties

         } else {
            String msg = "property - Wrong property: " + pName;
            Util_Logger.error(logger, msg);
         }

      }

   }

   /**
    * This method is used to convert a list in the form
    * item;item;item into a vector of strings.
    * <p>
    * @param value - the string items separated by ;
    * @returns a Vector of strings
    */
   private static Vector<String> buildVector(String value) {

      int index = 0;
      Vector<String> vector = new Vector<String>();
      boolean control = true;
      do {
         index = value.indexOf(";");
         if (index >= 0) {
            String fileType = value.substring(0, index);
            if (!fileType.equals(";")) { // v1.4.3 // java 1.6
               if (fileType.trim().length() > 0) {
                  vector.add(fileType.trim());
               }
            }
            value = value.substring(index + 1);
         } else {
            control = false;
         }

      } while (control);

      if (!value.equals(";")) {
         if (value.trim().length() > 0) {
            vector.add(value.trim());
         }
      }
      return vector;

   }

   /** This routine does a range check of an integer
    * @param property - the property that is being translated String
    * @param pValue the string value of the property eg property=value
    * @param lowLimit the lowest legal integer value limit
    * @param highLimit the highest legal integer value
    * @return the integer
    */
   private static int checkInt(String property, String pValue, int lowLimit, int highLimit) {
      int intValue = 0;
      try {
         intValue = Integer.parseInt(pValue);
         if (intValue < lowLimit) {
            throw new IllegalArgumentException("Value " + intValue + " below " + lowLimit);
         }
         if (intValue > highLimit) {
            throw new IllegalArgumentException("Value " + intValue + " above " + highLimit);
         }
      } catch (Exception e) {
         String msg = "property - " + property + " invalid ";
         Util_Logger.error(logger, msg, e);
      }
      return intValue;
   }
}
