/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.xml;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.Util_Logger;

import org.apache.log4j.Logger;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

/** This class is used to parse and validate ACK file generated
 *  by MSATS. There will be three possibilities:
 *  Positive ACK file, Negative ACK File and 111 Code ACK File
 *  <p>
 * @author Kevin Jin
 * version 1.0 - 15 Sep 2003 - Creation
 * @version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 */
public class AckFileParser {

   private static Logger logger = Logger.getLogger(AckFileParser.class);

   /**
    * Constructor
    */
   public AckFileParser() {}

   /**
    * This method is used to validate ACK file.
    * There will be three possibilities for ACK files:
    * Positive ACK, Negitive ACK and 111 Code ACK
    *
    * @param fileName - The ACK file to be validated  String
    * @return ackFileStatus defined in the Constants.java String
    */
   public String validateAckFile(String fileName) {
      // If there are problems when parsing the ACK file, we will
      // set it as negative to ask user manually processing
      String returnStatus = Constants.NEGATIVE_ACK;

      try {

         BatcherDOMParser parser = new BatcherDOMParser();
         Document doc = parser.createDocument(fileName, true);

         if (isAcceptMessageStatus(doc)) {
            returnStatus = Constants.POSITIVE_ACK;
         } else {
            if (isResendMessageStatus(doc)) {
               returnStatus = Constants.RESEND_ACK;
            }
         }
      } catch (Exception e) {
         Util_Logger.warning(logger, "Can not parse ACK file - " + fileName, e);
      }
      return returnStatus;

   }

   // This method is used to check "Status" in "MessageAcknowledgement"
   private boolean isAcceptMessageStatus(Document document) {

      boolean retStatus = false;

      // get elements that match
      NodeList elements = document.getElementsByTagName("MessageAcknowledgement");
      if ((elements == null) || !(elements.getLength() > 0)) {
         // No MessageAcknowledgement tag.
         return retStatus;
      }

      int elementCount = elements.getLength();
      for (int i = 0; i < elementCount; i++) {
         Element element = (Element)elements.item(i);
         NamedNodeMap attributes = element.getAttributes();
         if (attributes.getNamedItem("status") != null) {

            int attributeCount = attributes.getLength();
            for (int j = 0; j < attributeCount; j++) {
               Attr attribute = (Attr)attributes.item(j);
               if (attribute.getNodeName().equals("status")) {
                  if (attribute.getNodeValue().equals("Accept")) {
                     retStatus = true;
                  } else {
                     retStatus = false;
                  }
                  break;
               }
            }
         }
      }

      return retStatus;
   }

   // This method is used to check "Code" in "Event" under "MessageAcknowledgement"
   private boolean isResendMessageStatus(Document document) {

      boolean retStatus = false;

      // get elements that match
      NodeList elements = document.getElementsByTagName("Code");
      if ((elements == null) || !(elements.getLength() > 0)) {
         // No Code tag.
         return retStatus;
      }

      for (int j = 0; j < elements.getLength(); j++) {
         Element element = (Element)elements.item(j);

         if ("111".equalsIgnoreCase(element.getFirstChild().getNodeValue())) {
            retStatus = true;
            break;
         }

      }

      return retStatus;
   }

   // This method is used to check "Status" in "TransactionAcknowledgement"
   // Currently this is not used
   private boolean isAcceptTransactionStatus(Document document) {

      boolean retStatus = false;

      // get elements that match
      NodeList elements = document.getElementsByTagName("TransactionAcknowledgement");

      // is there anything to do?
      if ((elements == null) || !(elements.getLength() > 0)) {
         return retStatus; // no TransactionAcknowledgement tag.
      }

      int elementCount = elements.getLength();
      for (int i = 0; i < elementCount; i++) {
         Element element = (Element)elements.item(i);
         NamedNodeMap attributes = element.getAttributes();
         if (attributes.getNamedItem("status") != null) {
            int attributeCount = attributes.getLength();
            for (int j = 0; j < attributeCount; j++) {
               Attr attribute = (Attr)attributes.item(j);
               if (attribute.getNodeName().equals("status")) {
                  if (attribute.getNodeValue().equals("Accept")) {
                     retStatus = true;
                  } else {
                     retStatus = false;
                  }
                  break;
               }
            }
         }
      }

      return retStatus;
   }

}
