/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 * 10-Feb-2015     Warwick Mills      5252 Metering Batcher fails when duplicate file exists
 */

package com.nemmco.participantbatcher.xml;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.File;
import java.io.FileWriter;

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * This class is used to generate Acknowledgement (ack) file.
 *
 * @author Kevin Jin
 *  version 1.0   - 16 Sep 2003 Creation
 *  version 1.3   -  4-Oct-2004 log4j
 *  version 1.3.1 - 16-Nov-2004 Make -ve ack work when Schema invalid, header missing and zip uncompress error
 *  version 1.3.3 - 16-Feb-2005 Only insist on FROM, TO and TRANSACTIONGROUP in header for +ve ack
 *  version 1.4.1 -  9-Sep-2005 Make the ID used in an ACK file to be unique, extend format to .....ssSS
 *  vsesion 1.4.2 - 19-Jan-2006 Fix the special codes in a comphrensive way
 *  version 1.4.3 - 22-Mar-2006 Fix two date formats in ack file to be 24 date format - defect 3776
 *  version 1.4.4 -  5-Apr-2006 javaDoc and coding standards
 *  version 1.5.1 - 19-Jan-2007 Fix the replaceAllCodes routine to work with '&'
 *  version 1.5.2 - 13-Apr-2007 Use fixSpecialCodes everywhere
 *  version 1.5.3 - 10-Jul-2009 5796 Replace the schema version in the in both the "ackschemanamespace"
 *                              and  "ackschemalocation" property strings with the schema version in the zip.
 *  version 2.0.2 - 19-Nov-2010 More peer review
 * @version 2.0.7 - 10-Feb-2015 Generate unique message IDs
 *  version 2.0.13 - 21-Dec-2020 Market Element is a mandatory element for 5MS changes. This fix is for Issue 3481 - SIT3C5 - Market element is missing in the mack generated in participant inbox by participant and instance batcher.Issue 3481 - SIT3C5 - Market element is missing in the mack generated in participant inbox by participant and instance batcher.
 */

public class AckFileWriter {

   private static Logger logger = Logger.getLogger(AckFileWriter.class);
   private static Date lastMessageDate = new Date();
   private static int messageSuffix = 1;
   private static SimpleDateFormat idFormat = new SimpleDateFormat("yyyyMMddHHmmssSS");
   private static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

   /**
    * Constructor
    */
   public AckFileWriter() {}

   /**
    * This method is used to generate the acknowledgement (ack) files
    *
    * @param fileName qualified file name for the acknowledgemwnt file String
    * @param zpdw ZipFileInfo information retrieved from original XML message in object
    */
   public boolean generateAck(String fileName, ZipFileInfo zpdw) {

      String messageId = null;
      String messageDate = null;
      File ackFile = new File(fileName);
      FileWriter fw;

      try {

         // create the new .ack file
         if (!ackFile.exists()) {
            // this should not happened - it always does
            if (!ackFile.createNewFile())
               throw new Exception("Creation of ack file failed " + fileName);
         }

         fw = new FileWriter(ackFile.getAbsolutePath(), false);

         // make sure the header fields have been derived from input file
         boolean headerOk = (!zpdw.getToName().isEmpty()) && (!zpdw.getFromName().isEmpty()) && (!zpdw.getTransGroup().isEmpty());

         // if schema valid and header info is complete -> create +ve ack
         if (headerOk) {
            synchronized (lastMessageDate) {
               long now = System.currentTimeMillis();
               if (now == lastMessageDate.getTime()) {
                  messageSuffix++;
               } else {
                  lastMessageDate.setTime(now);
                  messageSuffix = 1;
               }
               messageId = "BH-" + idFormat.format(lastMessageDate) + "-" + messageSuffix;
               messageDate = dateFormat.format(lastMessageDate);
            }

            // Start to generate .ack file
            generateEnvlopeHeader("aseXML", fw, zpdw.getTransGroup(), zpdw.getSchemaVersion());
            generateHeaderElement(fw, zpdw, messageId, messageDate);
            generateAckElement(fw, zpdw, messageId, messageDate);
            fw.write("</ase:aseXML>\n");

         } else if (!zpdw.isValid()) { // want to -ve ack

            // Start to generate .ack file "schema invalid"
            generateEnvlopeHeader("Event", fw, zpdw.getTransGroup(), zpdw.getSchemaVersion());
            generateNackSchema(fw, zpdw);
            fw.write("</ase:Event>\n");

         } else {

            // Start to generate .ack file "header invalid"
            generateEnvlopeHeader("Event", fw, zpdw.getTransGroup(), zpdw.getSchemaVersion());
            generateNackHeader(fw);
            fw.write("</ase:Event>\n");
         }
         // finish up
         fw.flush();
         fw.close();
         return true;

      } catch (Exception e) {
         Util_Logger.warning(logger, "Fail to generate Ack" + fileName, e);
         return false;
      }
   }

   /**
    * This method is used to generate a -ve acknowledgement (ack) file.
    *
    * @param fileName - qualified file name for the acknowledgemwnt file String
    */
   public boolean generateZipNack(String fileName) {

      File ackFile = new File(fileName);
      FileWriter fw;

      try {
         // create the new .ack file
         if (!ackFile.exists()) {
            // this should not happened - it always does
            if (!ackFile.createNewFile())
               throw new Exception("Creation of ack file failed " + fileName);
         }

         fw = new FileWriter(ackFile.getAbsolutePath(), false);

         // Start to generate .ack file
         generateEnvlopeHeader("Event", fw, "", "");
         generateNackZip(fw);
         fw.write("</ase:Event>\n");

         // finish up
         fw.flush();
         fw.close();
         return true;

      } catch (Exception e) {
         Util_Logger.warning(logger, "Fail to generate Ack" + fileName, e);
         return false;
      }
   }

   // This method is used to genererate the Envelope Header of the .ack XML file
   private void generateEnvlopeHeader(String firstTag, FileWriter out, String zipTransGroup, String zipSchemaVersion) throws Exception {
      String ackSchemaNamespace = Constants.getConstant().getAckSchemaNamespace();
      String ackSchemaLocation = Constants.getConstant().getAckSchemaLocation();
      Vector<String> B2BFileTypes = Constants.getConstant().getB2BFileTypes();

      if (!B2BFileTypes.isEmpty() && zipTransGroup != null && zipTransGroup.length() > 0 && (B2BFileTypes.contains(zipTransGroup.toLowerCase())) && zipSchemaVersion != null && zipSchemaVersion.length() > 0) {
         String[] ackSchemaNamespaceComponents = ackSchemaNamespace.split(":");
         String propSchemaVersion = ackSchemaNamespaceComponents[2];

         ackSchemaNamespace = ackSchemaNamespace.replaceFirst(propSchemaVersion, zipSchemaVersion);
         ackSchemaLocation = ackSchemaLocation.replaceAll(propSchemaVersion, zipSchemaVersion);
      }

      String namespace = "<ase:" + firstTag + " xmlns:ase=\"" + ackSchemaNamespace + "\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " + "xsi:schemaLocation=\"" + ackSchemaNamespace + " " + ackSchemaLocation + "\">\n";

      out.write("<?xml version=\"1.0\" ?>\n");
      out.write(namespace);

   }

   // This method is used to generate the Header element of the .ack XML file
   private void generateHeaderElement(FileWriter out, ZipFileInfo zpdw, String messageId, String messageDate) throws Exception {

      out.write("   <Header>\n");
      out.write("      <From" + generateDescription(zpdw.getToDesc()) + ">" + zpdw.getToName() + "</From>\n");
      out.write("      <To" + generateDescription(zpdw.getFromDesc()) + ">" + zpdw.getFromName() + "</To>\n");
      out.write("      <MessageID>" + messageId + "</MessageID>\n");
      out.write("      <MessageDate>" + messageDate + "</MessageDate>\n");
      out.write("      <TransactionGroup>" + zpdw.getTransGroup() + "</TransactionGroup>\n");
      if (zpdw.getPriority() != "") {
         out.write("      <Priority>" + zpdw.getPriority() + "</Priority>\n");
      }
      if (zpdw.getSecurityContext() != "") {
         out.write("      <SecurityContext>" + zpdw.getSecurityContext() + "</SecurityContext>\n");
      }
		out.write("      <Market>" + Constants.MARKET + "</Market>\n");

      out.write("   </Header>\n");

   }

   // this method produces description="string" if string is not null, otherwise produces null V1.3.2
   private String generateDescription(String optionalString) {
      if (!optionalString.isEmpty()) {
         return " description=\"" + fixSpecialCodes(optionalString) + "\""; // v1.5.2
      } else {
         return "";
      }
   }

   // This method is used to generate the Acknowledgements element in the .ack XML file
   private void generateAckElement(FileWriter out, ZipFileInfo zpdw, String messageId, String messageDate) throws Exception {

      // We are only generate the MessageAcknowledgement element.
      out.write("   <Acknowledgements>\n");
      if (zpdw.isValid()) {
         // positive acknowledge
         out.write("      <MessageAcknowledgement initiatingMessageID=\"" + zpdw.getInitMsgId() + "\" receiptDate=\"" + messageDate + "\" receiptID=\"" + messageId + "\" status=\"Accept\">\n");
      } else {
         // negative acknowledge
         out.write("      <MessageAcknowledgement initiatingMessageID=\"" + zpdw.getInitMsgId() + "\" receiptDate=\"" + messageDate + "\" receiptID=\"" + messageId + "\" status=\"Reject\">\n");
         out.write("         <Event class=\"Message\" severity=\"Fatal\">\n");
         out.write("            <Code description=\"Schema validation failure\">2</Code>\n");
         out.write("            <Explanation>" + fixSpecialCodes(zpdw.getErrorMessage()) + "</Explanation>\n"); // v1.5.2
         out.write("         </Event>\n");
      }
      out.write("      </MessageAcknowledgement>\n");
      out.write("   </Acknowledgements>\n");

   }

   // This method is used to generate the -ve Acknowledgements element in the .ack XML file
   private void generateNackSchema(FileWriter out, ZipFileInfo zpdw) throws Exception {

      // We only generate the Nack CODE and EXPLANATION elements.
      // For a negative acknowledgement, there will be an Event
      // element associating with the MessageAcknowledgement element to
      // discribe the error reasons.

      // negative acknowledge - schema is NOT valid

      out.write("            <Code description=\"Schema validation failure\">2</Code>\n");
      out.write("            <Explanation>" + fixSpecialCodes(zpdw.getErrorMessage()) + "</Explanation>\n"); // v1.5.1

   }

   // This method is used to generate the -ve Acknowledgements element in the .ack XML file
   private void generateNackHeader(FileWriter out) throws Exception {

      // We only generate the Nack CODE and EXPLANATION elements.
      // For a negative acknowledgement, there will be an Event
      // element associating with the MessageAcknowledgement element to
      // discribe the error reasons.

      out.write("            <Code description=\"Header Incorrect\">7</Code>\n");
      out.write("            <Explanation>Header Incorrect</Explanation>\n");

   }

   // This method is used to generate the -ve Acknowledgements element in the .ack XML file
   private void generateNackZip(FileWriter out) throws Exception {

      // We only generate the Nack CODE and EXPLANATION elements.
      // For a negative acknowledgement, there will be an Event
      // element associating with the MessageAcknowledgement element to
      // discribe the error reasons.

      // negative acknowledge - schema is NOT valid

      out.write("            <Code description=\"Uncompression failure\">5</Code>\n");
      out.write("            <Explanation>Invalid zip file</Explanation>\n");

   }

   // the use of > or < in a string needs to be recoded before use in xml V1.4.2
   private String replaceAllCodes(String message, char singleCharacter, String replacementCharacter) {

      StringBuffer ib = new StringBuffer(message);
      StringBuffer ob = new StringBuffer();
      int j = 0;

      // replace & with &amp;
      for (int i = 0; i < ib.length(); i++) {
         if (ib.charAt(i) == singleCharacter) {
            ob.insert(j, '&');
            ob.insert(j + 1, replacementCharacter);
            j = j + replacementCharacter.length() + 1;
         } else {
            ob.insert(j, ib.charAt(i));
            j = j + 1;
         }
      }

      return ob.toString();

   }

   private String fixSpecialCodes(String message) {

      message = replaceAllCodes(message, '&', "amp;");
      message = replaceAllCodes(message, '\'', "apos;");
      message = replaceAllCodes(message, '"', "quot;");
      message = replaceAllCodes(message, '>', "gt;");
      message = replaceAllCodes(message, '<', "lt;");

      return message;
   }

   public static void main(String[] args) {

      AckFileWriter afw = new AckFileWriter();
      String mess = "&This < > is the ''mes>sage\" & ok?";
      String dog = afw.fixSpecialCodes(mess);
      System.out.println(mess);
      System.out.println(dog);

   }

}
