/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.xml;

import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.FileInputStream;
import java.io.InputStream;

import java.util.Vector;

import org.apache.log4j.Logger;
import org.apache.xerces.impl.Constants;
import org.apache.xerces.parsers.StandardParserConfiguration;
import org.apache.xerces.parsers.XMLGrammarPreparser;
import org.apache.xerces.util.SymbolTable;
import org.apache.xerces.util.XMLGrammarPoolImpl;
import org.apache.xerces.xni.grammars.Grammar;
import org.apache.xerces.xni.grammars.XMLGrammarDescription;
import org.apache.xerces.xni.parser.XMLInputSource;

import org.w3c.dom.Document;

import org.xml.sax.InputSource;

/**
 * This class is implemented to ensure Xerces Schema Caching feature
 * being utilized.
 * <p>
 * It is based on NEMMCO's proof of concept - MSATSDOMHelper.java
 * <p>
 * @author Kevin Jin
 * version 1.0 - 15 Sep 2003 - Creation
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * version 1.5.4 15-Feb-2008 Fixed using new Xerces library and Java 1.6.0
 * version 2.0.0 23-Jun-2010 Bob Jones Remove unused imports, variables and commented out code.
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 */

public class BatcherDOMParser {

   private String errorMessage = "";
   private static XMLGrammarPoolImpl grammarPool;

   /** Property identifier: grammar pool. */
   public static final String GRAMMAR_POOL = Constants.XERCES_PROPERTY_PREFIX + Constants.XMLGRAMMAR_POOL_PROPERTY;

   /** Namespaces feature id (http://xml.org/sax/features/namespaces). */
   protected static final String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";

   /** Validation feature id (http://xml.org/sax/features/validation). */
   protected static final String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";

   /** Schema validation feature id (http://apache.org/xml/features/validation/schema). */
   protected static final String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";

   /** Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
   protected static final String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";

   private static Logger logger = Logger.getLogger(BatcherDOMParser.class);

   /**
    * Constructor
    */
   public BatcherDOMParser() {}

   static {

      /* Xerces Parser v2.4.0 :: SC :: 22/05/03 */
      SymbolTable sym = new SymbolTable();

      XMLGrammarPreparser preparser = new XMLGrammarPreparser(sym);
      grammarPool = new XMLGrammarPoolImpl();

      preparser.registerPreparser(XMLGrammarDescription.XML_SCHEMA, null);

      preparser.setProperty(GRAMMAR_POOL, grammarPool);
      preparser.setFeature(NAMESPACES_FEATURE_ID, true);
      preparser.setFeature(VALIDATION_FEATURE_ID, true);
      // note we can set schema features just in case...
      preparser.setFeature(SCHEMA_VALIDATION_FEATURE_ID, true);
      preparser.setFeature(SCHEMA_FULL_CHECKING_FEATURE_ID, true);

      Vector<String> schemas = com.nemmco.participantbatcher.util.Constants.getConstant().getSchemaCaches();
      try {

         for (int i = 0; i < schemas.size(); i++) {
            Grammar g = preparser.preparseGrammar(XMLGrammarDescription.XML_SCHEMA, stringToXIS(schemas.elementAt(i)));
            Util_Logger.info(logger, "Load schema " + schemas.elementAt(i));
            // we don't really care about g; grammarPool will take care of everything.
         }
      } catch (Exception e) {
         // Can not create the parser
         Util_Logger.error(logger, "Can not initialize BatcherDOMParser", e);
      }

      // Now we have a grammar pool; just
      // build a configuration and we're on our way!

   }

   /**
    * This method is used to retrieve the parser error message.
    *
    * @return String errorMessage
    */
   public String getErrorMessage() {
      return this.errorMessage;
   }

   /**
    * This method is used to parse and validate the XML file.
    *
    * @param  fileName - the XML file  name String
    * @param  validate - TRUE validate / FALSE not validate boolean
    * @return Document - the XML document
    * @throws Exception
    */
   public Document createDocument(String fileName, boolean validate) throws Exception {

      Document document;

      // read the xml file
      FileInputStream fis = new FileInputStream(fileName);
      try {
         document = createDocument((InputStream)fis, validate);
      } finally {
         fis.close();
      }

      return document;
   }

   /**
    * This method is used to parse and validate the XML file.
    *
    * @param inputStream InputStream
    * @param  validate - TRUE validate / FALSE not validate boolean
    * @return Document - the XML document
    * @throws Exception
    */
   public Document createDocument(InputStream inputStream, boolean validate) throws Exception {

      Document document = null;

      org.apache.xerces.parsers.DOMParser dpXerces;
      BatcherErrorHandler errorHandler = new BatcherErrorHandler();
      InputSource is = new InputSource(inputStream);

      StandardParserConfiguration config = new StandardParserConfiguration();
      config.setProperty(GRAMMAR_POOL, grammarPool);
      dpXerces = new org.apache.xerces.parsers.DOMParser(config);

      if (validate) {
         // now must reset features, unfortunately:
         dpXerces.setFeature("http://xml.org/sax/features/namespaces", true);
         dpXerces.setFeature("http://xml.org/sax/features/validation", true);
         dpXerces.setFeature("http://apache.org/xml/features/validation/schema", true);
         dpXerces.setFeature("http://apache.org/xml/features/validation/schema-full-checking", true);
      }

      dpXerces.setErrorHandler(errorHandler);

      dpXerces.parse(is);

      errorHandler = (BatcherErrorHandler)dpXerces.getErrorHandler();
      this.errorMessage = errorHandler.getErrorMessages();

      document = dpXerces.getDocument();
      is = null;
      dpXerces = null;
      return document;

   }

   private static XMLInputSource stringToXIS(String uri) {
      return new XMLInputSource(null, uri, null);
   }
}
